const St = imports.gi.St;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const Lang = imports.lang;
const GLib = imports.gi.GLib;
//const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;
const Util = imports.misc.util;
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Extension.imports.lib;

const schema = "org.gnome.shell.extensions.CpuMemInfo";

function init(extensionMeta)
{
    return new CpuMemInfo(extensionMeta, schema);
}

function CpuMemInfo(extensionMeta, schema)
{
    this.init(extensionMeta, schema);
}

CpuMemInfo.prototype =
{
    schema: null,
    settings: null,
    paramPosition: null,
    paramTimerVal: null,
    paramDisplayIcons: null,
    paramEnableCpu: null,
    paramEnableMem: null,
    paramUnitMem: null,
    paramEnableDisk: null,
    paramEnableNet: null,

    icon: null,
    labelCpu: null,
    labelUnitCpu: null,
    iconCpu: null,
    labelMem: null,
    labelUnitMem: null,
    iconMem: null,
    labelDisk: null,
    labelUnitDisk: null,
    iconDisk: null,
    labelNet: null,
    labelUnitNet: null,
    iconNet: null,
    box: null,
    button: null,

    timer: null,

    cpuTotalOld: null,
    cpuIdleOld: null,
    diskTotalReadOld: null,
    diskTotalWriteOld: null,
    diskIdleOld: null,
    netDownTotalOld: null,
    netUpTotalOld: null,
    netIdleOld: null,
   

    init: function(extensionMeta, schema)
    {
	this.schema = schema;
    },

    onParamChanged: function()
    {
	let paramPositionOld = this.paramPosition;
	this.paramPosition = this.settings.get_string("position");
	this.paramTimerVal = this.settings.get_int("timer-val");
	this.paramDisplayIcons = this.settings.get_boolean("display-icons");
	this.paramEnableCpu = this.settings.get_boolean("enable-cpu");
	this.paramEnableMem = this.settings.get_boolean("enable-mem");
	this.paramEnableDisk = this.settings.get_boolean("enable-disk");
	this.paramEnableNet = this.settings.get_boolean("enable-net");
	this.paramUnitMem = this.settings.get_string("unit-mem");

	if (this.timer != null)
            Mainloop.source_remove(this.timer);
	this.timer = Mainloop.timeout_add(this.paramTimerVal, Lang.bind(this, this.update));

	if (this.paramDisplayIcons && (this.paramEnableCpu || this.paramEnableMem || this.paramEnableDisk  || this.paramEnableNet))
	    this.icon.hide();
	else
	    this.icon.show();

	if (this.paramEnableCpu)
	{
	    this.labelCpu.show();
	    this.labelUnitCpu.show();
	    if (this.paramDisplayIcons)
		this.iconCpu.show();
	    else
		this.iconCpu.hide();
	}
	else
	{
	    this.labelCpu.hide();
	    this.labelUnitCpu.hide();
	    this.iconCpu.hide();
	}

	if (this.paramEnableMem)
	{
	    this.labelMem.show();
	    this.labelUnitMem.show();
	    if (this.paramDisplayIcons)
		this.iconMem.show();
	    else
		this.iconMem.hide();
	}
	else
	{
	    this.labelMem.hide();
	    this.labelUnitMem.hide();
	    this.iconMem.hide();
	}

	if (this.paramEnableDisk)
	{
	    this.labelDisk.show();
	    this.labelUnitDisk.show();
	    if (this.paramDisplayIcons)
		this.iconDisk.show();
	    else
		this.iconDisk.hide();
	}
	else
	{
	    this.labelDisk.hide();
	    this.labelUnitDisk.hide();
	    this.iconDisk.hide();
	}

	if (this.paramEnableNet)
	{
	    this.labelNet.show();
	    this.labelUnitNet.show();
	    if (this.paramDisplayIcons)
		this.iconNet.show();
	    else
		this.iconNet.hide();
	}
	else
	{
	    this.labelNet.hide();
	    this.labelUnitNet.hide();
	    this.iconNet.hide();
	}

	if (this.paramUnitMem != "Go" && this.paramUnitMem != "Mo" && this.paramUnitMem != "%")
	{
	    this.paramUnitMem = "Go";
	    this.settings.set_string("unit-mem", this.paramUnitMem);
	}
	this.labelUnitMem.set_text(this.paramUnitMem);

	if (this.paramPosition != null && this.paramPosition != paramPositionOld)
	{
	    if (paramPositionOld != null)
	    {
		if (paramPositionOld == "left")
		    Main.panel._leftBox.remove_child(this.button);
		else if (paramPositionOld == "center")
		    Main.panel._centerBox.remove_child(this.button);
		else
		    Main.panel._rightBox.remove_child(this.button);
	    }

	    if (this.paramPosition == "left")
		Main.panel._leftBox.insert_child_at_index(this.button, Main.panel._leftBox.get_children().length);
	    else if (this.paramPosition == "center")
		Main.panel._centerBox.insert_child_at_index(this.button, Main.panel._leftBox.get_children().length);
	    else
	    {
		Main.panel._rightBox.insert_child_at_index(this.button, 0);
		this.paramPosition = "right";
		this.settings.set_string("position", this.paramPosition);
	    }
	}

	this.update();
    },
    
    enable: function()
    {
        this.icon = new St.Icon(
        {
//	    gicon: Gio.icon_new_for_string(meta.path + "/processor.png"),
//	    icon_type: St.IconType.SYMBOLIC,
	    icon_name: "utilities-system-monitor-symbolic",
	    icon_size: 16,
	    style_class: "cmi-icon"
        });

	this.labelCpu = new St.Label({
	    style_class: "cmi-cpu-label"
	});

	this.labelUnitCpu = new St.Label({
	    text: "%",
	    style_class: "cmi-cpu-unit-label"
	});

        this.iconCpu = new St.Icon(
        {
	    icon_name: "computer-symbolic",
	    icon_size: 16,
	    style_class: "cmi-cpu-icon"
        });

	this.labelMem = new St.Label({
	    style_class: "cmi-mem-label"
	});

	this.labelUnitMem = new St.Label({
	    style_class: "cmi-mem-unit-label"
	});

        this.iconMem = new St.Icon(
        {
	    icon_name: "emblem-system-symbolic",
	    icon_size: 16,
	    style_class: "cmi-mem-icon"
        });

	this.labelDisk = new St.Label({
	    style_class: "cmi-disk-label"
	});

	this.labelUnitDisk = new St.Label({
	    text: "Mo/s",
	    style_class: "cmi-disk-unit-label"
	});

        this.iconDisk = new St.Icon(
        {
	    icon_name: "drive-harddisk-symbolic",
	    icon_size: 16,
	    style_class: "cmi-disk-icon"
        });

	this.labelNet = new St.Label({
	    style_class: "cmi-net-label"
	});

	this.labelUnitNet = new St.Label({
	    style_class: "cmi-net-unit-label"
	});

        this.iconNet = new St.Icon(
        {
	    icon_name: "emblem-shared-symbolic",
	    icon_size: 16,
	    style_class: "cmi-net-icon"
        });

	this.box = new St.BoxLayout();
        this.box.add_actor(this.icon);
        this.box.add_actor(this.labelCpu);
        this.box.add_actor(this.labelUnitCpu);
        this.box.add_actor(this.iconCpu);
        this.box.add_actor(this.labelMem);
        this.box.add_actor(this.labelUnitMem);
        this.box.add_actor(this.iconMem);
        this.box.add_actor(this.labelDisk);
        this.box.add_actor(this.labelUnitDisk);
        this.box.add_actor(this.iconDisk);
        this.box.add_actor(this.labelNet);
        this.box.add_actor(this.labelUnitNet);
        this.box.add_actor(this.iconNet);

	this.button = new St.Button({
	    style_class: "panel-button"
	});
        this.button.connect("clicked", this.clicked);
        this.button.set_child(this.box);

	let settings = new Lib.Settings(this.schema);
	this.settings = settings.getSettings();
	this.settings.connect("changed::position", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::timer-val", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::display-icons", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::enable-cpu", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::enable-mem", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::unit-mem", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::enable-disk", Lang.bind(this, this.onParamChanged));
	this.settings.connect("changed::enable-net", Lang.bind(this, this.onParamChanged));

	this.onParamChanged();
    },

    disable: function()
    {
	if (this.paramPosition == "left")
	    Main.panel._leftBox.remove_child(this.button);
	else if (this.paramPosition == "center")
	    Main.panel._centerBox.remove_child(this.button);
	else
	    Main.panel._rightBox.remove_child(this.button);
	this.paramPosition = null;

        Mainloop.source_remove(this.timer);
    },

    clicked: function()
    {
	let app = global.log(Shell.AppSystem.get_default().lookup_app("gnome-system-monitor.desktop"));

        if (app != null)
            app.activate();
        else
            Util.spawn(["gnome-system-monitor"]);
    },

    update: function()
    {
	if (this.paramEnableCpu)
	    this.updateCpu();

	if (this.paramEnableMem)
	    this.updateMem();

	if (this.paramEnableDisk)
	    this.updateDisk();

	if (this.paramEnableNet)
	    this.updateNet();

	return true;
    },

    updateCpu: function()
    {
	let cpuTotal = 0, cpuIdle;

	let fileStat = GLib.file_get_contents("/proc/stat");
        let linesStat = ("" + fileStat[1]).split("\n");

        for (let i = 0; i < linesStat.length; i++)
	{
	    let values;

	    if (linesStat[i].match(/^cpu /))
	    {
                values = linesStat[i].match(/^cpu\s*(.*)$/)[1].split(" ");

		cpuIdle = values[3];

		for (let i = 0; i < values.length; i++)
		    cpuTotal += parseInt(values[i]);
	    }
	}

	if ((this.cpuTotalOld == null && this.cpuIdleOld == null)
	    || (this.cpuTotalOld == cpuTotal && this.cpuIdleOld == cpuIdle))

	{
	    this.cpuTotalOld = cpuTotal - 1;
	    this.cpuIdleOld = cpuIdle - 1;
	}

	let diffCpuTotal = cpuTotal - this.cpuTotalOld;
	let diffCpuIdle = cpuIdle - this.cpuIdleOld;
	
	let cpuValue = Math.round(100 * (diffCpuTotal - diffCpuIdle) / diffCpuTotal);

	this.cpuTotalOld = cpuTotal;
	this.cpuIdleOld = cpuIdle;

        this.labelCpu.set_text(cpuValue.toString());
    },

    updateMem: function()
    {
       	let memTotal, memFree, memBuffers, memCached;

	let fileMeminfo = GLib.file_get_contents("/proc/meminfo");
	let linesMeminfo = ("" + fileMeminfo[1]).split("\n");

	for (let i = 0; i < linesMeminfo.length; i++)
	{
	    let values;

	    if (linesMeminfo[i].match(/^MemTotal/))
	    {
		values = linesMeminfo[i].match(/^MemTotal:\s*([^ ]*)\s*([^ ]*)$/);
		    
		memTotal = values[1];
	    }
	    else if (linesMeminfo[i].match(/^MemFree/))
	    {
		values = linesMeminfo[i].match(/^MemFree:\s*([^ ]*)\s*([^ ]*)$/);
		    
		memFree = values[1];
	    }
	    else if (linesMeminfo[i].match(/^Buffers/))
	    {
		values = linesMeminfo[i].match(/^Buffers:\s*([^ ]*)\s*([^ ]*)$/);
		    
		memBuffers = values[1];
	    }
	    else if (linesMeminfo[i].match(/^Cached/))
	    {
		values = linesMeminfo[i].match(/^Cached:\s*([^ ]*)\s*([^ ]*)$/);
		    
		memCached = values[1];
	    }	    
        }

	let memUsed = memTotal - memFree - memBuffers - memCached;

	if (this.paramUnitMem == "%")
	    this.labelMem.set_text(Math.round(memUsed / memTotal * 100).toString());
	else
	{
	    let precision = 1;
	    let unitFactor = 1024;

	    if (this.paramUnitMem == "Go")
	    {
		unitFactor = 1048576;

		if (memTotal >= 10 * unitFactor)
		    precision = 10;
		else
		    precision = 100;
	    }

	    memUsed = Math.round(precision * memUsed / unitFactor) / precision;
	    memTotal = Math.round(precision * memTotal / unitFactor) / precision;

	    this.labelMem.set_text(memUsed.toString() + " / " + memTotal.toString());
	}
    },

    updateDisk: function()
    {
	let fileDiskstats = GLib.file_get_contents("/proc/diskstats");
	let linesDiskstats = ("" + fileDiskstats[1]).split("\n");

	let diskReadTotal = 0;
	let diskWriteTotal = 0;
	let diskIdle = new Date().getTime() / 1000;

	for (let i = 0; i < linesDiskstats.length; i++)
	{
	    let values;

	    if (linesDiskstats[i].match(/^\s*\d+\s*\d+\s[a-z]+\s/))
	    {
		let values = linesDiskstats[i].match(/^\s*\d+\s*\d+\s[a-z]+\s(.*)$/)[1].split(" ");
		
		diskReadTotal += parseInt(values[2]);
		diskWriteTotal += parseInt(values[6]);
	    }
	}

	if ((this.diskReadTotalOld == null && this.diskWriteTotalOld == null && this.diskIdleOld == null)
	    || (this.diskIdleOld == diskIdle))
	{
	    this.diskReadTotalOld = diskReadTotal;
	    this.diskWriteTotalOld = diskWriteTotal;
	    this.diskIdleOld = diskIdle - 1;
	}
	
	let diskRead = Math.round((this.diskReadTotalOld - diskReadTotal) / (this.diskIdleOld - diskIdle) / 2 / 1024);
	let diskWrite = Math.round((this.diskWriteTotalOld - diskWriteTotal) / (this.diskIdleOld - diskIdle) / 2 / 1024);
    
	this.diskReadTotalOld = diskReadTotal;
	this.diskWriteTotalOld = diskWriteTotal;
	this.diskIdleOld = diskIdle;

	this.labelDisk.set_text(diskRead.toString() + " / " + diskWrite.toString());
    },

    updateNet: function()
    {
	let fileNetDev = GLib.file_get_contents("/proc/net/dev");
	let linesNetDev = ("" + fileNetDev[1]).split("\n");

	let netDownTotal = 0;
	let netUpTotal = 0;
	let netIdle = new Date().getTime() / 1000;

	for (let i = 0; i < linesNetDev.length; i++)
	{
	    let values;

	    if (linesNetDev[i].match(/^\s[^:]*:/))
	    {
		let values = linesNetDev[i].match(/^\s*[^:]*:\s+([0-9]+)\s+[0-9]+\s+[0-9]+\s+[0-9]+\s+[0-9]+\s+[0-9]+\s+[0-9]+\s+[0-9]+\s+([0-9]+)/);
		netDownTotal += parseInt(values[1]);
		netUpTotal += parseInt(values[2]);
	    }
	}

	if ((this.netDownTotalOld == null && this.netUpTotalOld == null && this.netIdleOld == null)
	    || (this.netIdleOld == netIdle))
	{
	    this.netDownTotalOld = netDownTotal;
	    this.netUpTotalOld = netUpTotal;
	    this.netIdleOld = netIdle - 1;
	}
	
	let netDown = Math.round((this.netDownTotalOld - netDownTotal) / (this.netIdleOld - netIdle) / 1024);
	let netUp = Math.round((this.netUpTotalOld - netUpTotal) / (this.netIdleOld - netIdle) / 1024);

	if (netDown > 1024 || netUp > 1024)
	{
	    this.labelUnitNet.set_text("Mo/s");

	    if (netDown > 1024 * 99 || netUp > 1024 * 99)
	    {
		netDown = Math.round(netDown / 1024);
		netUp = Math.round(netUp / 1024);
	    }
	    else
	    {
		netDown = Math.round(netDown / 1024 * 10) / 10;
		netUp = Math.round(netUp / 1024 * 10) / 10;
	    }
	}
	else
	    this.labelUnitNet.set_text("Ko/s");

	this.netDownTotalOld = netDownTotal;
	this.netUpTotalOld = netUpTotal;
	this.netIdleOld = netIdle;

	this.labelNet.set_text(netDown.toString() + " / " + netUp.toString());
    }
}
