const Gtk = imports.gi.Gtk;
const Lang = imports.lang;

const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Extension.imports.lib;

const schema = "org.gnome.shell.extensions.CpuMemInfo";

function init()
{

}

function buildPrefsWidget()
{
    let prefs = new Prefs(schema);

    return prefs.buildPrefsWidget();
}

function Prefs(schema)
{
    this.init(schema);
}

Prefs.prototype =
{
    settings: null,

    init: function(schema)
    {
	let settings = new Lib.Settings(schema);
	
	this.settings = settings.getSettings();
    },

    changePosition: function(object, position)
    {
	this.settings.set_string("position", position);
    },

    changeTimerVal: function(object, valueTimerVal)
    {
	this.settings.set_int("timer-val", valueTimerVal.get_value() * 1000);
    },

    changeDisplayIcons: function(object, pspec)
    {
	this.settings.set_boolean("display-icons", object.active);
    },

    changeEnableCpu: function(object, pspec)
    {
	this.settings.set_boolean("enable-cpu", object.active);
    },

    changeEnableMem: function(object, pspec)
    {
	this.settings.set_boolean("enable-mem", object.active);
    },

    changeEnableDisk: function(object, pspec)
    {
	this.settings.set_boolean("enable-disk", object.active);
    },

    changeUnitMem: function(object, unitMem)
    {
	this.settings.set_string("unit-mem", unitMem);
    },

    changeEnableNet: function(object, pspec)
    {
	this.settings.set_boolean("enable-net", object.active);
    },

    buildPrefsWidget: function()
    {
	let frame = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, border_width: 10});
	let label = new Gtk.Label({ label: "<b>Global</b>", use_markup: true, xalign: 0});
	let vbox = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL, margin_left: 20});



	let hboxPosition = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelPosition = new Gtk.Label({label: "Position", xalign: 0});
	let value1Position = new Gtk.RadioButton({label: "left"});
	let value2Position = new Gtk.RadioButton({label: "center", group: value1Position});
	let value3Position = new Gtk.RadioButton({label: "right", group: value1Position});
	if (this.settings.get_string("position") == 'left')
	    value1Position.set_active(true);
	else if (this.settings.get_string("position") == 'center')
	    value2Position.set_active(true);
	else
	    value3Position.set_active(true);
	value1Position.connect('toggled', Lang.bind(this, this.changePosition, "left"));
	value2Position.connect('toggled', Lang.bind(this, this.changePosition, "center"));
	value3Position.connect('toggled', Lang.bind(this, this.changePosition, "right"));

	hboxPosition.pack_start(labelPosition, true, true, 0);
	hboxPosition.add(value1Position);
	hboxPosition.add(value2Position);
	hboxPosition.add(value3Position);
	vbox.add(hboxPosition);



	let hboxTimerVal = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelTimerVal = new Gtk.Label({label: "Update Interval (seconds) ", xalign: 0});
	let valueTimerVal = new Gtk.Adjustment({lower: 1, upper: 300, step_increment: 1});
	let value2TimerVal = new Gtk.HScale({digits: 0, adjustment: valueTimerVal, value_pos: Gtk.PositionType.RIGHT});
	value2TimerVal.set_value(Math.round(this.settings.get_int("timer-val") / 1000));

	hboxTimerVal.add(labelTimerVal);
	hboxTimerVal.pack_end(value2TimerVal, true, true, 0);

	vbox.add(hboxTimerVal);

	let hboxButtonTimerVal = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let buttonTimerVal = new Gtk.Button({ label: "Apply Interval" });
	buttonTimerVal.connect("clicked", Lang.bind(this, this.changeTimerVal, valueTimerVal));

	hboxButtonTimerVal.add(buttonTimerVal);
	vbox.add(hboxButtonTimerVal);



	let hboxDisplayIcons = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelDisplayIcons = new Gtk.Label({label: "Display Icons", xalign: 0});
	let valueDisplayIcons = new Gtk.Switch({active: this.settings.get_boolean("display-icons")});
	valueDisplayIcons.connect('notify::active', Lang.bind(this, this.changeDisplayIcons));

	hboxDisplayIcons.pack_start(labelDisplayIcons, true, true, 0);
	hboxDisplayIcons.add(valueDisplayIcons);
	vbox.add(hboxDisplayIcons);



	let hboxEnableCpu = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelEnableCpu = new Gtk.Label({label: "Enable Cpu", xalign: 0});
	let valueEnableCpu = new Gtk.Switch({active: this.settings.get_boolean("enable-cpu")});
	valueEnableCpu.connect('notify::active', Lang.bind(this, this.changeEnableCpu));

	hboxEnableCpu.pack_start(labelEnableCpu, true, true, 0);
	hboxEnableCpu.add(valueEnableCpu);
	vbox.add(hboxEnableCpu);



	let hboxEnableMem = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelEnableMem = new Gtk.Label({label: "Enable Mem", xalign: 0});
	let valueEnableMem = new Gtk.Switch({active: this.settings.get_boolean("enable-mem")});
	valueEnableMem.connect('notify::active', Lang.bind(this, this.changeEnableMem));

	hboxEnableMem.pack_start(labelEnableMem, true, true, 0);
	hboxEnableMem.add(valueEnableMem);
	vbox.add(hboxEnableMem);



	let hboxEnableDisk = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelEnableDisk = new Gtk.Label({label: "Enable Disk", xalign: 0});
	let valueEnableDisk = new Gtk.Switch({active: this.settings.get_boolean("enable-disk")});
	valueEnableDisk.connect('notify::active', Lang.bind(this, this.changeEnableDisk));

	hboxEnableDisk.pack_start(labelEnableDisk, true, true, 0);
	hboxEnableDisk.add(valueEnableDisk);
	vbox.add(hboxEnableDisk);



	let hboxUnitMem = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelUnitMem = new Gtk.Label({label: "Unit Mem", xalign: 0});
	let value1UnitMem = new Gtk.RadioButton({label: "Go"});
	let value2UnitMem = new Gtk.RadioButton({label: "Mo", group: value1UnitMem});
	let value3UnitMem = new Gtk.RadioButton({label: "%", group: value1UnitMem});
	if (this.settings.get_string("unit-mem") == 'Go')
	    value1UnitMem.set_active(true);
	else if (this.settings.get_string("unit-mem") == 'Mo')
	    value2UnitMem.set_active(true);
	else
	    value3UnitMem.set_active(true);
	value1UnitMem.connect('toggled', Lang.bind(this, this.changeUnitMem, "Go"));
	value2UnitMem.connect('toggled', Lang.bind(this, this.changeUnitMem, "Mo"));
	value3UnitMem.connect('toggled', Lang.bind(this, this.changeUnitMem, "%"));

	hboxUnitMem.pack_start(labelUnitMem, true, true, 0);
	hboxUnitMem.add(value1UnitMem);
	hboxUnitMem.add(value2UnitMem);
	hboxUnitMem.add(value3UnitMem);
	vbox.add(hboxUnitMem);



	let hboxEnableNet = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL});
	let labelEnableNet = new Gtk.Label({label: "Enable Net", xalign: 0});
	let valueEnableNet = new Gtk.Switch({active: this.settings.get_boolean("enable-net")});
	valueEnableNet.connect('notify::active', Lang.bind(this, this.changeEnableNet));

	hboxEnableNet.pack_start(labelEnableNet, true, true, 0);
	hboxEnableNet.add(valueEnableNet);
	vbox.add(hboxEnableNet);



	frame.add(label);
	frame.add(vbox);
	frame.show_all();
	
	return frame;
    }
}
